# coding=utf-8
# Copyright (C) 2020-2022
# Author: César Román
# Contact: cesar@thecesrom.dev
#
# Licensed under the MIT License.
#
# https://github.com/thecesrom/incendium/blob/HEAD/LICENSE
#

"""incendium.

incendium is a package that extends and wraps some functions from
Ignition Scripting API.

For more information, please refer to the Wiki.
https://github.com/thecesrom/incendium/wiki
"""
